use std::io::Write;
use serde::{Deserialize, Serialize};
use crate::{DockerTagsFetcher, QueryArgs};

#[derive(Serialize, Deserialize)]
struct DockerResult {
    content_type: Option<String>,
    creator: u64,
    full_size: u64,
    id: u64,
    last_updated: Option<String>,
    last_updater: u64,
    last_updater_username: Option<String>,
    media_type: Option<String>,
    name: Option<String>,
    repository: u32,
    tag_last_pulled: Option<String>,
    tag_last_pushed: Option<String>,
    tag_status: Option<String>,
    v2: bool,
    images: Vec<DockerImage>,
}

#[derive(Serialize, Deserialize)]
struct DockerImage {
    architecture: Option<String>,
    digest: Option<String>,
    features: Option<String>,
    last_pulled: Option<String>,
    last_pushed: Option<String>,
    os: Option<String>,
    os_features: Option<String>,
    os_version: Option<String>,
    size: u64,
    status: Option<String>,
    variant: Option<String>,
}

#[derive(Serialize, Deserialize)]
struct DockerResponse {
    count: u32,
    next: Option<String>,
    previous: Option<String>,
    results: Vec<DockerResult>,
}

pub struct DockerHubTagsFetcher {}

impl DockerHubTagsFetcher {
    pub fn new() -> DockerHubTagsFetcher {
        DockerHubTagsFetcher {}
    }
}

impl DockerTagsFetcher for DockerHubTagsFetcher {
    fn get_tags(&self, args: &QueryArgs) -> Vec<String> {
        let page = 0;
        let page_size = 1000;

        let namespace = args.namespace.as_str();
        let repository = args.repository.as_str();
        let mut url = format!("https://hub.docker.com/v2/namespaces/{namespace}/repositories/{repository}/tags?page={page}&page_size={page_size}");
        let mut results: Vec<DockerResult> = Vec::new();
        loop {
            let resp = reqwest::blocking::get(&url).unwrap();
            if resp.status().is_success() {
                let text = resp.text().unwrap();
                let mut response: DockerResponse = serde_json::from_str(&text).unwrap();
                results.append(&mut response.results);
                url = match response.next {
                    None => { break; }
                    Some(next) => { next }
                };
            } else {
                let _x = writeln!(std::io::stdout(), "Image not found: {}/{}", namespace, repository);
                break;
            };
        }
        let mut filtered = results.into_iter()
            .filter(|x| !x.images.is_empty())
            .collect::<Vec<DockerResult>>();
        if args.name.is_some() {
            let pat = args.name.clone().unwrap();
            filtered = filtered.into_iter().filter(|x| x.name.clone().unwrap().contains(&pat))
                .collect::<Vec<DockerResult>>();
        };
        if args.arch.is_some() {
            let pat = args.arch.clone().unwrap();
            filtered = filtered.into_iter().filter(|x| x.images.get(0).as_ref().unwrap().architecture.clone().unwrap().contains(&pat))
                .collect::<Vec<DockerResult>>();
        };
        let mut results = Vec::<String>::new();
        for x in &filtered {
            results.push(x.name.clone().unwrap());
        }
        results
    }
}